import {NaiveUiResolver} from "unplugin-vue-components/dist/resolvers.js";

import Components from 'unplugin-vue-components/vite';

import viteCompression from "vite-plugin-compression";

import {nodes, is_mac, zip_file} from "../nodes.js";

import AutoImport from "unplugin-auto-import/vite";

import {createHtmlPlugin} from "vite-plugin-html";

import Markdown from 'unplugin-vue-markdown/vite';

import {viteMockServe} from "vite-plugin-mock";

import viteVue from "@vitejs/plugin-vue";

import svgLoader from "vite-svg-loader";

import frameConfig from "./config.js";

import alone from "../alone.js";

export default async function (pathname, option = {}) {
    const then = await frameConfig(pathname, option);
    const plugin_vue = function (data) {
        return viteVue(alone.array_merge({include: [/\.vue$/, /\.md$/]}, data));
    }
    const plugin_compress = function (data) {
        return viteCompression(alone.array_merge({
            verbose: true,    // 默认即可
            disable: true,  //开启压缩(不禁用)，默认即可
            deleteOriginFile: false, //删除源文件
            threshold: 10240, //压缩前最小文件大小
            algorithm: 'gzip',  //压缩算法
            ext: '.gz', //文件类型
        }, data));
    }
    const plugin_svg = function (data) {
        return svgLoader(alone.array_merge({defaultImport: 'url'}, data));
    }
    const plugin_html = function (data) {
        const tags = alone.get_arr(data, 'inject.tags');
        const config = alone.array_merge(data, {minify: true, inject: {tags: []}});
        config.inject.tags = (alone.isset(tags) && alone.is_array(tags)) ? then.html_tag.concat(tags) : then.html_tag;
        config.entry = then.option.main;
        return createHtmlPlugin(config);
    }
    const plugin_md = function (data) {
        const config = {headEnabled: true, markdownItOptions: {html: true, linkify: true, typographer: true}};
        const include = [`${then.node.get_path('.')}/*.md`, `${then.node.get_path('.')}/**/*.md`];
        const includes = alone.get_arr(data, 'include');
        config.include = (alone.isset(includes) && alone.is_array(includes)) ? include.concat(includes) : include;
        return Markdown(config);
    }
    const plugin_mock = function (data) {
        return viteMockServe(alone.array_merge({
            mockPath: then.env.mock,//存放 Mock 文件的路径。默认情况下，插件会在此路径下查找 Mock 文件
            watchFiles: true,//指示是否监视 Mock 文件的变化。如果设置为 true，当 Mock 文件发生变化时，Mock 服务器会自动更新。
            enable: true,//指示是否启用 Mock 功能。设置为 false 可以临时禁用 Mock。
            logger: false,//指示是否启用插件的日志记录。设置为 true 会在终端输出 Mock 请求的日志信息。
            cors: true //指示是否启用 CORS（跨域资源共享）。设置为 true 会在响应中添加 CORS 头部，以允许跨域请求。
        }, data || {}));
    }
    const plugin_auto = function (data) {
        return AutoImport(alone.array_merge({
            dts: then.env.dts.auto || false,
            imports: ['vue', {
                'naive-ui': [
                    'useDialog',
                    'useMessage',
                    'useNotification',
                    'useLoadingBar'
                ]
            }]
        }, data || {}));
    }
    const plugin_component = function (data) {
        return Components(alone.array_merge({
            dts: then.env.dts.component || false,
            extensions: ['vue', 'md'],
            include: [/\.vue$/, /\.vue\?vue/, /\.md$/],
            resolvers: [NaiveUiResolver()],
        }, data || {}));
    }
    const plugin_alone = function () {
        return {
            enforce: 'pre',
            name: 'alone-frame',
            config(config) {
                config.root = then.app_path;
                config.base = config.base || "./";
                config.publicDir = then.publicDir();
                config.resolve = config.resolve || {};
                config.define = alone.array_merge(config.define, then.define());
                config.resolve.alias = alone.array_merge(config.resolve.alias, then.alias());
                config.build = alone.array_merge(config.build, then.plugin_build());
                config.preview = alone.array_merge(config.preview, then.preview());
                config.server = alone.array_merge(config.server, then.server());
            },
            buildStart() {
                then.vue_route();
                console.log('path', `${is_mac() ? "file://" : ""}${then.app.get_path('.')}`);
            },
            closeBundle() {
                console.log('complete', `${is_mac() ? "file://" : ""}` + then.out_dir);
            },
            transform(code, file) {
                const beforeCode = `
            import MainLoadBefore from "alone-frame/frame/main";
            MainLoadBefore.status = {};
            await MainLoadBefore.load();
            `;
                const afterCode = `
            import MainLoadAfter from "alone-frame/frame/main";
            MainLoadAfter.mount();
            `;
                return then.plugin_transform_main(code, file, beforeCode, afterCode);
            }
        };
    }
    const plugin_build = function () {
        return {
            enforce: 'post',
            apply: 'build',
            name: 'alone-frame-zip',
            buildStart() {
                nodes(then.out_dir).del_dir_file('.', then.env.exclude);
            },
            closeBundle() {
                nodes(then.out_dir).delete_file('layui/lay.js');
                if (then.env.zip === true) {
                    zip_file(then.out_dir, alone.rtrim(then.out_dir, '/') + ".zip", function (size, unit, file) {
                        console.log('zip file', file);
                        console.log('zip size', `${size}${unit}`);
                    });
                }
            }
        };
    }
    return [
        (option.vue !== false && plugin_vue(option.vue)),
        (option.compress !== false && plugin_compress(option.compress)),
        (option.svg !== false && plugin_svg(option.svg)),
        (option.html !== false && plugin_html(option.html)),
        (option.md !== false && plugin_md(option.md)),
        ((option.mock !== false && then.env.mock) && plugin_mock(option.mock)),
        (option.auto !== false && plugin_auto(option.auto)),
        (option.component !== false && plugin_component(option.components)),
        plugin_alone(),
        plugin_build()
    ];
}